"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const chronoshift_1 = require("chronoshift");
const express_1 = require("express");
const plywood_1 = require("plywood");
const logger_1 = require("../../../common/logger/logger");
const queryable_data_cube_1 = require("../../../common/models/data-cube/queryable-data-cube");
const sources_1 = require("../../../common/models/sources/sources");
const datacube_guard_1 = require("../../utils/datacube-guard/datacube-guard");
const load_query_decorator_1 = require("../../utils/query-decorator-loader/load-query-decorator");
function plywoodRouter(settingsManager) {
    const logger = settingsManager.logger;
    const router = express_1.Router();
    router.post("/", async (req, res) => {
        const { dataSource, expression: expressionRaw, timezone } = req.body;
        const dataCube = req.body.dataCube || dataSource;
        if (typeof dataCube !== "string") {
            res.status(400).send({
                error: "must have a dataCube"
            });
            return;
        }
        let queryTimezone = null;
        if (typeof timezone === "string") {
            try {
                queryTimezone = chronoshift_1.Timezone.fromJS(timezone);
            }
            catch (e) {
                res.status(400).send({
                    error: "bad timezone",
                    message: e.message
                });
                return;
            }
        }
        let parsedExpression = null;
        try {
            parsedExpression = plywood_1.Expression.fromJS(expressionRaw);
        }
        catch (e) {
            res.status(400).send({
                error: "bad expression",
                message: e.message
            });
            return;
        }
        let sources;
        try {
            sources = await settingsManager.getSources();
        }
        catch (e) {
            res.status(400).send({ error: "failed to get sources" });
            return;
        }
        const myDataCube = sources_1.getDataCube(sources, dataCube);
        if (!myDataCube) {
            res.status(400).send({ error: "unknown data cube" });
            return;
        }
        if (!queryable_data_cube_1.isQueryable(myDataCube)) {
            res.status(400).send({ error: "un queryable data cube" });
            return;
        }
        if (!(datacube_guard_1.checkAccess(myDataCube, req.headers))) {
            res.status(403).send({ error: "access denied" });
            return;
        }
        const maxQueries = myDataCube.maxQueries;
        const decorator = load_query_decorator_1.loadQueryDecorator(myDataCube, settingsManager.anchorPath, logger);
        const expression = decorator(parsedExpression, req);
        try {
            const data = await myDataCube.executor(expression, { maxQueries, timezone: queryTimezone });
            const reply = {
                result: plywood_1.Dataset.isDataset(data) ? data.toJS() : data
            };
            res.json(reply);
        }
        catch (error) {
            logger.error(logger_1.errorToMessage(error));
            res.status(500).send({
                error: "could not compute",
                message: error.message
            });
        }
    });
    return router;
}
exports.plywoodRouter = plywoodRouter;
//# sourceMappingURL=plywood.js.map